//
//  ViewController.m
//  testGestures
//
//  Created by shawn li on 13-7-29.
//  Copyright (c) 2013年 bigbellshawn. All rights reserved.
//

#import "ViewController.h"

@interface ViewController ()
@property (nonatomic, strong) UIImageView *imageView;
@end

@implementation ViewController

@synthesize imageView = _imageView;

- (void)viewDidLoad
{
    [super viewDidLoad];
    /* step1: 声明和初始化一个手势识别器 */
    UITapGestureRecognizer *singleTap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handleSingleTap:)];
    UITapGestureRecognizer *doubleTap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handleDoubleTap:)];
    UITapGestureRecognizer *twoFingerTap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handleTwoFingerTap:)];
    // 坑：selector不小心忘了写冒号，导致找不到handleSwipe */
    UISwipeGestureRecognizer *swipeLeft = [[UISwipeGestureRecognizer alloc] initWithTarget:self action:@selector(handleSwipe:)];
    UISwipeGestureRecognizer *swipeRight = [[UISwipeGestureRecognizer alloc] initWithTarget:self action:@selector(handleSwipe:)];
    UIRotationGestureRecognizer *rotation = [[UIRotationGestureRecognizer alloc] initWithTarget:self action:@selector(handleRotation:)];
    UIPanGestureRecognizer *pan = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(handlePan:)];
    UIPinchGestureRecognizer *pinch = [[UIPinchGestureRecognizer alloc] initWithTarget:self action:@selector(handlePinch:)];
    UILongPressGestureRecognizer *longPress = [[UILongPressGestureRecognizer alloc] initWithTarget:self action:@selector(handleLongPress:)];
    
    /* step2：对手势识别器进行属性设定 */
    [doubleTap setNumberOfTapsRequired:2];
    // 坑：twoFingerTap在模拟器上不灵敏，有时候会识别成singleTap
    [twoFingerTap setNumberOfTouchesRequired:2];

    [swipeLeft setNumberOfTouchesRequired:1];
    [swipeLeft setDirection:UISwipeGestureRecognizerDirectionLeft];
    [swipeRight setNumberOfTouchesRequired:1];
    [swipeRight setDirection:UISwipeGestureRecognizerDirectionRight];
    
    // 坑：此部分代码无用 注：left、right、up、down都是独立的手势，没法一个swipe全代替，如果只有一个swipe默认向右划
    /*
    UISwipeGestureRecognizer *swipe = [[UISwipeGestureRecognizer alloc] initWithTarget:self action:@selector(handleSwipe:)];
    [swipe setNumberOfTouchesRequired:1];
    [swipe setDirection:UISwipeGestureRecognizerDirectionLeft | UISwipeGestureRecognizerDirectionRight | UISwipeGestureRecognizerDirectionUp | UISwipeGestureRecognizerDirectionDown];
     
    [self.view addGestureRecognizer:swipe];
     */
    
    
    /* step3：把手势识别器加到view中去 */
    [self.view addGestureRecognizer:singleTap];
    [self.view addGestureRecognizer:doubleTap];
    [self.view addGestureRecognizer:twoFingerTap];
    [self.view addGestureRecognizer:swipeLeft];
    [self.view addGestureRecognizer:swipeRight];
    [self.view addGestureRecognizer:rotation];
    [self.view addGestureRecognizer:pan];
    [self.view addGestureRecognizer:pinch];
    [self.view addGestureRecognizer:longPress];
    
    /* step4：出现冲突时，要设定优先识别顺序，目前只是doubleTap、swipe */
    [singleTap requireGestureRecognizerToFail:doubleTap];
    [singleTap requireGestureRecognizerToFail:twoFingerTap];
    [pan requireGestureRecognizerToFail:swipeRight];
    [pan requireGestureRecognizerToFail:swipeLeft];
    
    
    _imageView = [[UIImageView alloc] initWithFrame:CGRectMake(10, 120, 100, 40)];
    [self.view addSubview:_imageView];
    
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}


#pragma mark handle gestures methods

/* step5：去实现处理扑捉到手势之后的动作 */

/* 识别单击 */
- (void)handleSingleTap:(UITapGestureRecognizer *)gestureRecognizer {
    CGPoint location = [gestureRecognizer locationInView:self.view];
	[self drawImageForGestureRecognizer:gestureRecognizer atPoint:location underAdditionalSituation:@"singleTap"];
	
	[UIView animateWithDuration:0.5 animations:^{
        self.imageView.alpha = 0.0;
    }];
}

/* 识别双击 */
- (void)handleDoubleTap:(UITapGestureRecognizer *)gestureRecognizer {
    CGPoint location = [gestureRecognizer locationInView:self.view];
	[self drawImageForGestureRecognizer:gestureRecognizer atPoint:location underAdditionalSituation:@"doubleTap"];
	
	[UIView animateWithDuration:0.5 animations:^{
        self.imageView.alpha = 0.0;
    }];
}

/* 识别两个手指击 */
- (void)handleTwoFingerTap:(UITapGestureRecognizer *)gestureRecognizer {
    CGPoint location = [gestureRecognizer locationInView:self.view];
	[self drawImageForGestureRecognizer:gestureRecognizer atPoint:location underAdditionalSituation:@"twoFingerTap"];
	
	[UIView animateWithDuration:0.5 animations:^{
        self.imageView.alpha = 0.0;
    }];
}

/* 识别侧滑 */
- (void)handleSwipe:(UISwipeGestureRecognizer *)gestureRecognizer {
    CGPoint location = [gestureRecognizer locationInView:self.view];
	[self drawImageForGestureRecognizer:gestureRecognizer atPoint:location underAdditionalSituation:nil];
    
    if (gestureRecognizer.direction == UISwipeGestureRecognizerDirectionLeft) {
        location.x -= 220.0;
    }
    else if (gestureRecognizer.direction == UISwipeGestureRecognizerDirectionUp) {
        location.x -= 220.0;
    }
    else if (gestureRecognizer.direction == UISwipeGestureRecognizerDirectionDown) {
        location.x -= 220.0;
    }
    else{
        location.x += 220.0;
    }
    
	[UIView animateWithDuration:0.5 animations:^{
        self.imageView.alpha = 0.0;
        self.imageView.center = location;
    }];
}

/* 识别翻转 */
- (void)handleRotation:(UIRotationGestureRecognizer *)gestureRecognizer {
    
    CGPoint location = [gestureRecognizer locationInView:self.view];
    
    CGAffineTransform transform = CGAffineTransformMakeRotation([gestureRecognizer rotation]);
    self.imageView.transform = transform;
	[self drawImageForGestureRecognizer:gestureRecognizer atPoint:location underAdditionalSituation:nil];
    
    if (([gestureRecognizer state] == UIGestureRecognizerStateEnded) || ([gestureRecognizer state] == UIGestureRecognizerStateCancelled)) {
        [UIView animateWithDuration:0.5 animations:^{
            self.imageView.alpha = 0.0;
            self.imageView.transform = CGAffineTransformIdentity;
        }];
    }

}

/* 识别拖动 */
- (void)handlePan:(UIPanGestureRecognizer *)gestureRecognizer {
    CGPoint location = [gestureRecognizer locationInView:self.view];
	[self drawImageForGestureRecognizer:gestureRecognizer atPoint:location underAdditionalSituation:nil];
    //gestureRecognizer.view.center = CGPointMake(gestureRecognizer.view.center.x + location.x, gestureRecognizer.view.center.y + location.y);
    [gestureRecognizer setTranslation:location inView:self.view];
}

/* 识别放大缩小 */
- (void)handlePinch:(UIPinchGestureRecognizer *)gestureRecognizer {
    CGPoint location = [gestureRecognizer locationInView:self.view];
	[self drawImageForGestureRecognizer:gestureRecognizer atPoint:location underAdditionalSituation:nil];
    gestureRecognizer.view.transform = CGAffineTransformScale(gestureRecognizer.view.transform, gestureRecognizer.scale, gestureRecognizer.scale);  
    gestureRecognizer.scale = 1;
}

/* 识别长按 */
- (void)handleLongPress:(UILongPressGestureRecognizer *)gestureRecognizer {
    CGPoint location = [gestureRecognizer locationInView:self.view];
	[self drawImageForGestureRecognizer:gestureRecognizer atPoint:location underAdditionalSituation:nil];
    
}
#pragma mark -
#pragma mark Drawing the image view

/*
 Set the appropriate image for the image view for the given gesture recognizer, move the image view to the given point, then dispay the image view by setting its alpha to 1.0.
 */
- (void)drawImageForGestureRecognizer:(UIGestureRecognizer *)recognizer
atPoint:(CGPoint)centerPoint underAdditionalSituation:(NSString *)addtionalSituation{
    
	NSString *imageName;
    
    if ([recognizer isMemberOfClass:[UITapGestureRecognizer class]]) {
        if ([addtionalSituation isEqualToString:@"singleTap"]) {
            imageName = @"tap.gif";
        }
        else if ([addtionalSituation isEqualToString:@"doubleTap"]){
            imageName = @"tap2.gif";
        }
        else if ([addtionalSituation isEqualToString:@"twoFingerTap"]){
            imageName = @"twoFingerTap.gif";
        }
        else{
            imageName = @"error.png";
        }
    }
    else if ([recognizer isMemberOfClass:[UIRotationGestureRecognizer class]]) {
        imageName = @"rotation.gif";
    }
    else if ([recognizer isMemberOfClass:[UISwipeGestureRecognizer class]]) {
        imageName = @"swipe.gif";
    }
    else if ([recognizer isMemberOfClass:[UIPinchGestureRecognizer class]]) {
        imageName = @"pinch.gif";
    }
    else if ([recognizer isMemberOfClass:[UIPanGestureRecognizer class]]) {
        imageName = @"pan.gif";
    }
    else if ([recognizer isMemberOfClass:[UILongPressGestureRecognizer class]]) {
        imageName = @"longPress.gif";
    }
    else {
        imageName = @"error.png";
    }
    
	self.imageView.image = [UIImage imageNamed:imageName];
	self.imageView.center = centerPoint;
	self.imageView.alpha = 1.0;
}
@end
